# -*- coding: utf-8 -*-
# Copyright (c) 2009- Spyder Project Contributors
#
# Distributed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""
Breakpoints Plugin API.
"""

# Local imports
from spyder.plugins.breakpoints.plugin import BreakpointsActions
from spyder.plugins.breakpoints.widgets.main_widget import (
    BreakpointTableViewActions)
