# -*- coding: utf-8 -*-
#
# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""
Spyder application toolbar constants.
"""


# --- Constants
# ----------------------------------------------------------------------------
class ApplicationToolbars:
    File = 'file_toolbar'
    Run = 'run_toolbar'
    Debug = 'debug_toolbar'
    Main = 'main_toolbar'
    WorkingDirectory = 'working_directory_toolbar'


class MainToolbarSections:
    LayoutSection = 'layout_toolbar_section'
    ApplicationSection = 'application_toolbar_section'
