# -*- coding: utf-8 -*-

# Copyright © Spyder Project Contributors
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)

"""LSP related widgets."""

from .messagebox import ServerDisabledMessageBox
from .serversconfig import LSPServerTable
from .status import ClientStatus, LSPStatusWidget
